@@ -12,6 +12,8 @@ from strenum import LowercaseStrEnum
 from nef_pipelines.lib.util import exit_error, is_float, is_int, running_in_pycharm
 
 NEF_CATEGORY_ATTR = "__NEF_CATEGORY__"
+NEF_MOLECULAR_SYSTEM = "nef_molecular_system"
+NEF_METADATA = "nef_nmr_meta_data"
 
 UNUSED = "."
 
@@ -27,6 +29,31 @@ class SelectionType(LowercaseStrEnum):
         return self.value, str(other).upper()
 
 
+SELECTORS_LOWER = ", ".join(
+    [selector.lower() for selector in SelectionType.__members__]
+)
+
+
+class PotentialTypes(LowercaseStrEnum):
+    UNDEFINED = auto()
+    LOG_HARMONIC = "log-harmonic"
+    PARABOLIC = auto()
+    SQUARE_WELL_PARABOLIC = "square-well-parabolic"
+    SQUARE_WELL_PARABOLIC_LINEAR = "square-well-parabolic-linear"
+    UPPER_BOUND_PARABOLIC = "upper-bound-parabolic"
+    LOWER_BOUND_PARABOLIC = "lower-bound-parabolic"
+    UPPER_BOUND_PARABOLIC_LINEAR = "upper-bound-parabolic-linear"
+    LOWER_BOUND_PARABOLIC_LINEAR = "lower-bound-parabolic-linear "
+
+    # see https://github.com/irgeek/StrEnum/issues/9
+    def _cmp_values(self, other):
+        return self.value, str(other).upper()
+
+
+POTENTIAL_TYPES_LOWER = ", ".join(
+    [selector.lower() for selector in PotentialTypes.__members__]
+)
+
 # currently disabled as they add a dependency on pandas and numpy
 # def loop_to_dataframe(loop: Loop) -> DataFrame:
 #     """
@@ -79,9 +106,9 @@ def select_frames_by_name(
     exact=False,
 ) -> Tuple[Saveframe]:
     """
-    select frames  by names and wild cards, to avoid typing *s on the command line the match is greedy by default
-    if an exact match is not found for one of the frames first time we search will all the name selectors turned into
-    wild cards by surrounding them with the * as a fence so name_selector-> *name_selector*
+    select frames  by names and wild cards, to avoid typing *s on the command line the match is greedy by default,
+    if an exact match is not found for one of the frames first time we search again with all the name selectors
+    turned into wild cards by surrounding them with the * as a fence so name_selector-> *name_selector*
 
     :param frames: the list of frames or entry to search
     :param name_selectors: a single string or list of strings to use to match frame names, selectors can contain
@@ -136,7 +163,7 @@ def create_entry_from_stdin_or_exit() -> Entry:
                 lines = "".join(stdin_lines)
 
             if len(lines.strip()) == 0:
-                raise Exception("stdin is empty")
+                exit_error("stdin is empty")
             else:
                 entry = Entry.from_string(lines)
         else:
@@ -214,11 +241,14 @@ def select_frames(
     :return: a list of selected saveframes
     """
 
+    if not filters:
+        filters = ["*"]
+
     star_filters = [f"*{filter}*" for filter in filters]
     filters = list(filters)
     filters.extend(star_filters)
 
-    result = []
+    result = {}
     for frame in entry.frame_dict.values():
 
         accept_frame_category = any(
@@ -228,31 +258,28 @@ def select_frames(
 
         if (
             selector_type in (SelectionType.NAME, SelectionType.ANY)
-            and not accept_frame_name
+            and accept_frame_name
         ):
-            continue
+            result[frame.category, frame.name] = frame
 
         if (
             selector_type in (SelectionType.CATEGORY, SelectionType.ANY)
-            and not accept_frame_category
+            and accept_frame_category
         ):
-            continue
+            result[frame.category, frame.name] = frame
 
-        result.append(frame)
-
-    return result
+    return list(result.values())
 
 
 def read_entry_from_file_or_stdin_or_exit_error(file: Path) -> Entry:
     """
     read a star entry from stdin or a file or exit.
-    note 1. the stdin stream is cached
-         2. this exits with an error if stdin can't be read because its a terminal
+    note this exits with an error if stdin can't be read because its a terminal
 
     :param file:
     :return:
     """
-    if file is None:
+    if file is None or file == Path("-"):
         entry = create_entry_from_stdin_or_exit()
     else:
         try:
@@ -262,3 +289,21 @@ def read_entry_from_file_or_stdin_or_exit_error(file: Path) -> Entry:
         except IOError as e:
             exit_error(f"couldn't read from the file {file}", e)
     return entry
+
+
+def file_name_path_to_frame_name(path: str) -> str:
+    """
+    convert a filename or complete path to a name suitable for use as a frame name, this does the following mappings
+        . -> _
+        - -> _
+
+    :param file_name: the filename to convert
+    :return: a string suitable for use as a NEF framename
+    """
+    path = Path(path)
+
+    stem = str(path.stem)
+    stem = stem.replace("-", "_")
+    stem = stem.replace(".", "_")
+
+    return stem
