@@ -1,7 +1,7 @@
 import string
-import sys
 from collections import Counter
 from dataclasses import replace
+from pathlib import Path
 from textwrap import dedent
 from typing import Dict, Iterable, List, Optional, Tuple, Union
 
@@ -12,16 +12,17 @@ from nef_pipelines.lib.constants import NEF_UNKNOWN
 from nef_pipelines.lib.nef_lib import loop_row_namespace_iter
 
 # from nef_pipelines.lib.nef_lib import loop_to_dataframe
-from nef_pipelines.lib.structures import Linking, SequenceResidue
+from nef_pipelines.lib.structures import AtomLabel, Linking, SequenceResidue
 from nef_pipelines.lib.util import (
-    cached_stdin,
     chunks,
     exit_error,
+    get_display_file_name,
     is_int,
-    running_in_pycharm,
+    read_from_file_or_exit,
 )
 
 NEF_CHAIN_CODE = "chain_code"
+ANY_CHAIN = "_"
 
 
 def chain_code_iter(
@@ -33,6 +34,7 @@ def chain_code_iter(
 
     Args:
         user_chain_codes (str):  string of dot separated chain codes
+        exclude: chain codes to not include in the iteration
 
     Returns:
         Iterable[str] single codes
@@ -74,7 +76,6 @@ def get_linking(
     get the correct linking for residue in a standard sequenced chain
 
     Args:
-        target_index (int):  index in the sequence
         target_sequence (List[SequenceResidue]) : the sequence
         no_chain_start (List[str]): if the chain isn't in the list cap it at the start
         no_end (List[str]): if the chain isn't in the list cap it at the end
@@ -340,14 +341,17 @@ def count_residues(sequence_frame: Saveframe, chain_code: str) -> Dict[str, int]
     return result
 
 
-def get_sequence_or_exit() -> Optional[List[SequenceResidue]]:
+def get_sequence_or_exit(
+    file_name: Path = Path("-"),
+) -> Optional[List[SequenceResidue]]:
     """
-    read a sequence from a nef molecular system on stdin or exit
+    read a sequence from a nef molecular system on a file or stdin or exit, return a list of residues
 
+    file_name: a path to read the nef from Path('-') read from stdin
     :return: a list of parsed residues and chains, in the order they were read
     """
     try:
-        result = get_sequence()
+        result = get_sequence(file_name)
     except Exception as e:
         msg = "couldn't read sequence from nef input stream, either no stream or no nef molecular system frame"
         exit_error(msg, e)
@@ -359,43 +363,32 @@ def get_sequence_or_exit() -> Optional[List[SequenceResidue]]:
     return result
 
 
-def get_sequence() -> List[SequenceResidue]:
+def get_sequence(file_name: Path = Path("-")) -> List[SequenceResidue]:
     """
     read a sequence from a nef molecular system on stdin and return a list of residues
 
+    file_name: a path to read the nef from Path('-') read from stdin
     can raise Exceptions
 
     :return:a list of parsed residues and chains, in the order they were read
     """
 
-    result = []
+    text = read_from_file_or_exit(file_name, "sequence from NEF molecular system")
 
-    if sys.stdin.isatty():
-        exit_error(
-            "trying to read sequence from stdin but stdin is not a stream [did you forget to add a nef file to your "
-            "pipeline?]"
-        )
-
-    if running_in_pycharm():
-        exit_error(
-            "reading from stdin doesn't work in pycharm debug environment as there is no shell..."
-        )
-
-    stream = cached_stdin()
-
-    if stream:
-        text = "".join(stream)
+    display_file_name = get_display_file_name(file_name)
 
+    try:
         entry = Entry.from_string(text)
+    except Exception as e:
+        msg = f"couldn't parse text in {display_file_name} as NEF data"
+        exit_error(msg, e)
 
-        result = sequence_from_entry(entry)
-
-    return result
+    return sequence_from_entry(entry)
 
 
 def sequence_from_entry(entry):
 
-    result = None
+    result = []
     if entry is not None:
         frames = entry.get_saveframes_by_category("nef_molecular_system")
 
@@ -404,15 +397,31 @@ def sequence_from_entry(entry):
     return result
 
 
-def sequence_from_frame(frame: Saveframe) -> List[SequenceResidue]:
+def sequence_from_frame(
+    frame: Saveframe, chain_codes_to_select: Union[str, List[str]] = ANY_CHAIN
+) -> List[SequenceResidue]:
 
     """
     read sequences from a nef molecular system save frame
     can raise Exceptions
 
     :param frame: the save frame to read residues from, must have category nef molecular system
+    :param chain_code_to_select: the chain codes to select,t this can be either a string or list of strings,
+                                 any chain is selected using the instance of the constant string ANY_CHAIN [default]
     :return: a list of parsed residues and chains, in the order they were read
     """
+
+    if chain_codes_to_select is not ANY_CHAIN and isinstance(
+        chain_codes_to_select, str
+    ):
+        chain_codes_to_select = set(
+            [
+                chain_codes_to_select,
+            ]
+        )
+    elif chain_codes_to_select is not ANY_CHAIN:
+        chain_codes_to_select = set(chain_codes_to_select)
+
     residues = OrderedSet()
 
     if frame.category != "nef_molecular_system":
@@ -430,20 +439,21 @@ def sequence_from_frame(frame: Saveframe) -> List[SequenceResidue]:
 
     for line in loop:
         chain_code = line[chain_code_index]
-        sequence_code = line[sequence_code_index]
-        residue_name = line[residue_name_index]
-        linking = (
-            Linking[line[linking_index].upper()]
-            if line[linking_index] != NEF_UNKNOWN
-            else None
-        )
-        residue = SequenceResidue(
-            chain_code=chain_code,
-            sequence_code=sequence_code,
-            residue_name=residue_name,
-            linking=linking,
-        )
-        residues.append(residue)
+        if chain_codes_to_select is ANY_CHAIN or chain_code in chain_codes_to_select:
+            sequence_code = line[sequence_code_index]
+            residue_name = line[residue_name_index]
+            linking = (
+                Linking[line[linking_index].upper()]
+                if line[linking_index] != NEF_UNKNOWN
+                else None
+            )
+            residue = SequenceResidue(
+                chain_code=chain_code,
+                sequence_code=sequence_code,
+                residue_name=residue_name,
+                linking=linking,
+            )
+            residues.append(residue)
 
     return list(residues)
 
@@ -468,7 +478,7 @@ def sequence_3let_to_res(
 def get_chain_starts(residues: List[SequenceResidue]) -> Dict[str, int]:
     """
     from a list of residues get the lowest residue number for each chain ignoring any sequence codes that
-    can't be convetrted to an integer
+    can't be converted to an integer
 
     :param residues:  a list of residues from one ore more chains
     :return: a dictionary of chain starts by chain_code
@@ -536,3 +546,31 @@ def make_chunked_sequence_1let(
         row_strings.append(" ".join(row_string))
 
     return row_strings
+
+
+def residues_to_residue_name_lookup(
+    sequence: List[SequenceResidue],
+) -> Dict[Tuple[str, str], str]:
+    return {
+        (residue.chain_code, str(residue.sequence_code)): residue.residue_name
+        for residue in sequence
+    }
+
+
+def replace_chain_in_atom_labels(
+    atom_labels: List[AtomLabel], chain_code: str
+) -> List[AtomLabel]:
+    """
+    replace the chain in a list of AtomLabels, not this operation is not inplace new AtomLabels are returned
+
+    :param atom_labels: a list of AtomLabels
+    :param chain_code: the new chain_code
+    :return: the updated AtomLabels
+    """
+    result = []
+
+    for selection in atom_labels:
+        new_residue = replace(selection.residue, chain_code=chain_code)
+        result.append(replace(selection, residue=new_residue))
+
+    return result
