@@ -36,6 +36,28 @@ class PeakAxis:
     # comment: str
 
 
+@dataclass
+class DistanceRestraint:
+    atom_list_1: List[AtomLabel]
+    atom_list_2: List[AtomLabel]
+
+    target_distance: float
+    distance_minus: float
+    distance_plus: float
+
+
+@dataclass
+class DihedralRestraint:
+    atom_1: AtomLabel
+    atom_2: AtomLabel
+    atom_3: AtomLabel
+    atom_4: AtomLabel
+
+    target_angle: float
+    lower_limit: float
+    upper_limit: float
+
+
 @dataclass
 class PeakValues:
     serial: int
