@@ -4,6 +4,7 @@ import sys
 import traceback
 from argparse import Namespace
 from pathlib import Path
+from textwrap import dedent
 from typing import Dict, Iterator, List, Optional, TextIO, TypeVar, Union
 
 from cacheable_iter import iter_cache
@@ -23,6 +24,8 @@ from nef_pipelines.lib.header_lib import (
 )
 from nef_pipelines.lib.structures import LineInfo
 
+STDIN = Path("-")
+
 
 def _get_loop_by_category_or_none(frame: Saveframe, category: str) -> Loop:
 
@@ -177,6 +180,33 @@ def running_in_pycharm():
     return "PYCHARM_HOSTED" in os.environ
 
 
+def get_text_from_file_or_exit(file_name: Path) -> str:
+    """
+    get text from stdin or from a file argument
+
+    Args:
+        file_name (str): name of the file Path('-') indicates stdin
+
+    Returns:
+        str: text from file
+
+    """
+
+    result = None
+
+    if file_name == Path("-") or not file_name:
+        if sys.stdin.isatty():
+            exit_error("attempting to read from a terminal")
+        result = sys.stdin.read()
+    else:
+        try:
+            result = open(file_name).read()
+        except IOError as e:
+            exit_error(f"couldn't read from {file_name} because of an error", e)
+
+    return result
+
+
 def get_pipe_file(args: Namespace) -> Optional[TextIO]:
     """
     get an input on stdin or from an argument called pipe
@@ -299,6 +329,37 @@ def script_name(file: str) -> Path:
     return Path(file).name
 
 
+def read_from_file_or_exit(file_name: Path, target: str) -> str:
+    if file_name == STDIN and sys.stdin.isatty():
+        exit_error(
+            f"trying to open stdin to read {target}, but stdin is not a stream [did you forget to add a nef file to "
+            "your pipeline?]"
+        )
+    if running_in_pycharm():
+        exit_error(
+            "reading from stdin doesn't work in pycharm debug environment as there is no shell..."
+        )
+
+    display_file_name = get_display_file_name(file_name)
+
+    fh = sys.stdin
+
+    if not file_name == STDIN:
+        try:
+            fh = open(file_name)
+        except Exception as e:
+            msg = f"couldn't open {display_file_name} to read {target}"
+            exit_error(msg, e)
+
+    try:
+        text = fh.read()
+    except Exception as e:
+        msg = f"couldn't read {target} from {display_file_name}"
+        exit_error(msg, e)
+
+    return text
+
+
 def exit_error(msg, exception=None):
     """
     print an error message and exit error
@@ -312,7 +373,14 @@ def exit_error(msg, exception=None):
         traceback.print_exception(*exc_info, file=sys.stderr)
         print(file=sys.stderr)
 
-    print(f"ERROR: {msg}", file=sys.stderr)
+    msg = dedent(msg).split("\n")
+
+    print(file=sys.stderr)
+
+    print(f"ERROR: {msg[0]}", file=sys.stderr)
+
+    for line in msg[1:]:
+        print(f"       {line}", file=sys.stderr)
     print("exiting...", file=sys.stderr)
     sys.exit(EXIT_ERROR)
 
@@ -376,7 +444,7 @@ def is_int(value: str) -> bool:
     try:
         int(value)
         result = True
-    except ValueError:
+    except (ValueError, TypeError):
         pass
 
     return result
@@ -396,7 +464,7 @@ def is_float(value: str) -> bool:
     try:
         float(value)
         result = True
-    except ValueError:
+    except (ValueError, TypeError):
         pass
 
     return result
@@ -526,3 +594,13 @@ def parse_comma_separated_options(
             result.append(item)
 
     return result
+
+
+def end_with_ordinal(n):
+    return str(n) + {1: "st", 2: "nd", 3: "rd"}.get(
+        4 if 10 <= n % 100 < 20 else n % 10, "th"
+    )
+
+
+def get_display_file_name(file_name: Path) -> str:
+    return "stdin" if file_name == STDIN else file_name
